import logging

# Importing * is usually a death sin, but we justify it here by
# defining an __all__ in the relevant modules.
from .checkout import *
from .subscription import *
from .creditcheck import *
from .utils import configure
from requests import RequestException

logger = logging.Logger("sweetpay-sdk")



# -*- coding: utf-8 -*-

import json
import datetime
import logging
import os
from collections import namedtuple

from decimal import Decimal

import requests
from marshmallow import Schema
from requests import Session

from .errors import SweetpayError, BadDataError, InvalidParameterError, \
    InternalServerError, UnderMaintenanceError, UnauthorizedError, \
    NotFoundError, TimeoutError, RequestError

ResponseClass = namedtuple("ResponseClass",
                           ["response", "code", "data", "status"])


class BaseResource(object):
    """The base resource."""
    CLIENT_CLS = None
    api_token = None
    stage = None
    version = None
    timeout = None

    @classmethod
    def get_client(cls):
        if not cls.api_token or not cls.stage:
            raise ValueError("You must set an API token and decide whether "
                             "to use the stage environment or not before "
                             "using the SDK. Have a look at "
                             "`sweetpay.configure`")
        return cls.CLIENT_CLS(cls.api_token, cls.stage, cls.version)


def configure(api_token, stage, version=None, timeout=None):
    """Configure the API with global variables

    :param api_token: The API token provided by SweetPay.
    :param stage: A boolean indicating whether to use the stage
                  or production environment.
    :param version: An integer or string indicating which version
                    of the API to use. For example: 1
    :param timeout: The request timeout, defaults to
    :return: None
    """
    # We accomplish this by just setting some global variables
    # on the BaseResource
    BaseResource.api_token = api_token
    BaseResource.stage = stage
    BaseResource.version = version
    BaseResource.timeout = timeout


class BaseSchema(Schema):
    """The base marshmallow schema"""
    pass


class SweetpayJSONEncoder(json.JSONEncoder):
    """A custom JSON encoder to support custom types."""
    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            return obj.isoformat()
        elif isinstance(obj, datetime.date):
            return obj.strftime("%Y-%m-%d")
        elif isinstance(obj, datetime.timedelta):
            return (datetime.datetime.min + obj).time().isoformat()
        # Convert decimals to strings in order for it to
        # be money safe.
        elif isinstance(obj, Decimal):
            return str(obj)
        else:
            return super(SweetpayJSONEncoder, self).default(obj)


class BaseClient(object):
    """The super class used to create API clients."""

    def __init__(self, api_token, stage, version=None, timeout=None):
        """Initialize the checkout client used to talk to the checkout API.

        :param api_token: The authorization token to set in the
               Authorization header.
        :param stage: A boolean indicating if the stage server should
               be used or not. If set to False, the live server will be used.
        :param version: The version number of the API, defaults to 1.
        :param timeout: The request timeout. Defaults to 15 seconds.
        """
        self.stage = stage
        self.version = version or 1
        self.api_token = api_token
        self.timeout = timeout or 15

        # Setup the logger
        self.logger = logging.getLogger("sweetpay-sdk")

        # The session
        self.session = Session()
        self.session.headers.update({
            "Authorization": api_token, "Content-Type": "application/json",
            "Accept": "application/json", "User-Agent": "Python-SDK"
        })

    @property
    def stage_url(self):
        """Return the stage URL."""
        raise NotImplementedError("No URL for the stage server has "
                                  "been specified")

    @property
    def production_url(self):
        """Return the production URL."""
        raise NotImplementedError("No URL for the production server has "
                                  "been specified")

    @property
    def url(self):
        """Return the stage or production URL, depending on the settings."""
        if self.stage:
            return self.stage_url
        else:
            return self.production_url

    def build_url(self, *args):
        """Return a URL based on

        :param args: The arguments which will be used to build the path.
                For example: "path" and "to" creates the path "/path/to".
        :return: A complete URL as a string.
        """
        return os.path.join(self.url, *args)

    def make_request(self, url, method, respschema, reqschema=None,
                     params=None):
        """Make a request.

        :param params: The parameters to pass on to the marshmallow
                deserialization, and then pass on to the server.
        :param respschema: The schema to use for deserialization.
        :param params: The parameters passed by the client.
        :raise BadDataError: If the HTTP status code is 400
        :raise InvalidParameterError: If the HTTP status code is 422
        :raise InternalServerError: If an internal server error occurred
                                    at Sweetpay.
        :raise UnauthorizedError: If the API token was invalid.
        :raise NotFoundError: If the requested resource couldn't be found.
        :raise UnderMaintenanceError: If the server is currently
                                      under maintenance.
        :raise TimeoutError: If a request timeout occurred.
        :raise RequestError: If an unhandled request error occurred.
        :return: Return a dictionary with the keys "response",
                "status_code", "data" and "raw_data".
        """

        logger = self.logger

        # We want the method to be in lower-case for easier handling.
        method = method.lower()

        # Set default values for the request args and kwargs.
        reqargs = [url]
        reqkwargs = {"timeout": self.timeout}

        # Handle the method types appropriately
        if method == "get":
            logger.info("Sending request with method=GET to url=%s", url)
        elif method == "post":
            # TODO: Catch exception
            to_dump = reqschema().dump(params).data
            # Encode the data to JSON
            reqdata = SweetpayJSONEncoder().encode(to_dump)
            # Log the progress
            logger.info("Sending request with method=POST to url=%s and "
                        "parameters=%s", url, reqdata)
            # Set data as a kwarg to the request function
            reqkwargs["data"] = reqdata
        else:
            raise ValueError("Only GET and POST requests are allowed, "
                             "not method=%s", method)

        # Send the actual response
        response_func = getattr(self.session, method)
        try:
            resp = response_func(*reqargs, **reqkwargs)
        except requests.Timeout as e:
            # If the request timed out.
            raise TimeoutError("The request timed out", code=None, status=None,
                               respone=None, exc=e)
        except requests.RequestException as e:
            # If another request error occurred.
            raise RequestError("Could not send a request to the server",
                               code=None, status=None, response=None,
                               exc=e)
        # Shortcut to status code
        code = resp.status_code
        logger.info("With url=%s and method=%s, received status_code=%d "
                    "and body=%s", resp.status_code, url, method, resp.text)

        try:
            # Try to decode the (hopefully) JSON response
            data = resp.json()
        except (TypeError, ValueError):
            logger.error("Could not deserialize JSON for request "
                         "to url=%s, response=%s", url, resp.text)
            data = None
            status = None
        else:
            # TODO: Remove hack
            # We do this because the API sometimes return a string
            # as the data, and a string obviously can't have a status.
            if isinstance(data, dict):
                # Let's deserialize the data to Python objects.
                data = respschema.load(data).data

                # Now it's time to extract the status. If no status was
                # passed, an error will be raised as that shouldn't
                # be possible.
                try:
                    status = data["status"]
                except KeyError:
                    # We throw a general error here as this isn't
                    # supposed to happen.
                    raise SweetpayError("No status was passed from Sweetpay, "
                                        "even though the JSON was valid. This "
                                        "is most likely an error "
                                        "in Sweetpay's API", code=code,
                                        status=None, data=data, response=resp)
            else:
                status = None

        # Check if the response was successful.
        if code == 200:
            # Return a response class
            return ResponseClass(response=resp, code=resp.status_code,
                                 data=data, status=status)

        # The standard kwargs to send to the exception when raised
        exc_kwargs = {"code": code, "response": resp, "status": status,
                      "data": data}
        # Bad Data
        if code == 400:
            raise BadDataError("The data passed to the server "
                               "contained bad data. This most likely means "
                               "that you missed to send some parameters, "
                               "response data={0}".format(data),
                               **exc_kwargs)
        # Unauthorized
        elif code == 401:
            raise UnauthorizedError("The passed API token was invalid",
                                    **exc_kwargs)
        # Not Found
        elif code == 404:
            raise NotFoundError("The resource you were looking for couldn't "
                                "be found", **exc_kwargs)
        # Unprocessable Entity
        elif code == 422:
            raise InvalidParameterError("You passed in an invalid "
                                        "parameter or missed a parameter"
                                        "parameter, response data={0}".format(data),
                                        **exc_kwargs)
        # Internal Server Error
        elif code == 500:
            raise InternalServerError("An internal server occurred",
                                      **exc_kwargs)
        # Under Maintenance
        elif code == 503:
            raise UnderMaintenanceError("The server is currently under"
                                        "maintenance and can't be contacted",
                                        **exc_kwargs)
        # Something else happened, just throw a general error.
        else:
            raise SweetpayError("Something went wrong in the request",
                                **exc_kwargs)

    def __repr__(self):
        return u"<{0}: stage={1}, version={2}>".format(type(self).__name__,
                                                       self.stage,
                                                       self.version)



# -*- coding: utf-8 -*-


class SweetpayError(Exception):

    # The JSON decoded data from the server. None if the JSON
    # couldn't be decoded.
    data = None

    # The requests response object returned from the server.
    # None if the request failed.
    response = None

    # The HTTP status code as an integer. None if the request failed.
    code = None

    # The status extracted from the payload. None if the JSON
    # couldn't be decoded or if no status for some reason wasn't
    # returned from the server.
    status = None

    def __init__(self, msg, data, response, code, status):
        Exception.__init__(self, msg)
        self.data = data
        self.response = response
        self.code = code
        self.status = status


class BadDataError(SweetpayError):
    pass


class InvalidParameterError(SweetpayError):
    pass


class InternalServerError(SweetpayError):
    pass


class UnauthorizedError(SweetpayError):
    pass


class NotFoundError(SweetpayError):
    pass


class UnderMaintenanceError(SweetpayError):
    pass


class RequestError(SweetpayError):

    # This is a link to the requests exception that triggered
    # this error. Check this attribute if you want to see what kind
    # of request error occurred (e.g. a timeout).
    exc = None

    def __init__(self, *args, exc=None, **kwargs):
        SweetpayError.__init__(self, *args, **kwargs)
        self.exc = exc


class TimeoutError(RequestError):
    pass



import json

from marshmallow import fields
from .utils import BaseClient, BaseSchema, BaseResource

__all__ = ["SubscriptionClient", "deserialize_callback_data", "Subscription"]


class NameSchema(BaseSchema):
    """The Schema representing the Name object"""
    first = fields.String()
    last = fields.String()
    org = fields.String()


class AddressSchema(BaseSchema):
    """The Schema representing the Address object"""
    street = fields.String()
    zip = fields.String()
    city = fields.String()
    region = fields.String()
    country = fields.String()
    care_of = fields.String(dump_to="careOf", load_from="careOf")


class CustomerSchema(BaseSchema):
    """The Schema representing the Customer object"""
    phone = fields.String()
    ssn = fields.String()
    org = fields.String()
    email = fields.String()
    name = fields.Nested(NameSchema)
    address = fields.Nested(AddressSchema)


class SubscriptionSchema(BaseSchema):
    """The Schema representing the Subscription object"""
    currency = fields.String()
    amount = fields.Decimal()
    state = fields.String()
    created_at = fields.DateTime(load_from="createdAt", load_only=True)

    nextExecutionAt = fields.Date(load_from="nextExecutionAt", load_only=True)
    last_execution_at = fields.Date(load_from="lastExecutionAt",
                                    load_only=True)
    executions = fields.Integer()
    merchant_id = fields.String(load_from="merchantId", load_only=True)

    interval = fields.String()
    starts_at = fields.Date(dump_to="startsAt", load_from="startsAt")
    ends_at = fields.Date(dump_to="endsAt", load_from="endsAt")
    max_executions = fields.Integer(dump_to="maxExecutions",
                                    load_from="maxExecutions")
    execution_delay = fields.Integer(dump_to="executionDelay",
                                     load_from="executionDelay")

    merchant_subscription_id = fields.String(dump_to="merchantSubscriptionId",
                                             load_from="merchantSubscriptionId"
                                             )
    merchant_item_id = fields.String(dump_to="merchantItemId",
                                     load_from="merchantItemId")
    number_of_items = fields.Float(dump_to="numberOfItems",
                                   load_from="numberOfItems")
    item_unit = fields.String(dump_to="itemUnit", load_from="itemUnit")
    amount_per_item = fields.String(dump_to="amountPerItem",
                                    load_from="itemUnit")

    subscription_id = fields.Integer(load_from="subscriptionId",
                                     dump_to="subscriptionId")
    billing = fields.Nested(CustomerSchema, load_only=True)
    customer = fields.Nested(CustomerSchema, load_only=True)


class EnvelopeSchema(BaseSchema):
    """The Schema representing the Envelope object"""
    created_at = fields.DateTime(load_from="createdAt")
    status = fields.String()
    payload = fields.Nested(SubscriptionSchema)
    version = fields.String()


class SearchEnvelopeSchema(BaseSchema):
    """The Schema representing the Envelope object"""
    created_at = fields.DateTime(load_from="createdAt")
    status = fields.String()
    payload = fields.Nested(SubscriptionSchema, many=True)
    version = fields.String()


class CallbackEnvelope(BaseSchema):
    """The Schema representing the CallbackEnvelope object"""
    created_at = fields.DateTime(load_from="createdAt")
    sent_at = fields.DateTime(load_from="sentAt")
    callback_id = fields.Integer(load_from="callbackId")
    version = fields.String()
    event = fields.String()
    payload = fields.Nested(SubscriptionSchema)


class CreateSubscriptionRequestSchema(BaseSchema):
    """The Schema representing the CreateSessionRequest object"""
    country = fields.String()
    merchant_id = fields.String(dump_to="merchantId")
    language = fields.String()
    merchant_customer_id = fields.String(dump_to="merchantCustomerId")

    starts_at = fields.Date(dump_to="startsAt")
    ends_at = fields.Date(dump_to="endsAt")
    max_executions = fields.Integer(dump_to="maxExecutions")

    currency = fields.String()
    amount = fields.Decimal()

    comment = fields.String()
    vat = fields.Decimal()
    merchant_source = fields.String(dump_to="merchantSource")

    ssn = fields.String()
    org = fields.String()
    phone = fields.String()
    email = fields.String()
    name = fields.Nested(NameSchema)
    address = fields.Nested(AddressSchema)
    interval = fields.String()

    billing_phone = fields.String(dump_to="billingPhone")
    billing_email = fields.String(dump_to="billingEmail")
    billing_name = fields.Nested(NameSchema, dump_to="billingName")
    billing_address = fields.Nested(AddressSchema, dump_to="billingAddress")

    delivery_phone = fields.String(dump_to="deliveryPhone")
    delivery_email = fields.String(dump_to="deliveryEmail")
    delivery_name = fields.Nested(NameSchema, dump_to="deliveryName")
    delivery_address = fields.Nested(AddressSchema, dump_to="deliveryAddress")

    merchant_order_id = fields.String(dump_to="merchantOrderId")


class UpdateSubscriptionSchema(BaseSchema):
    starts_at = fields.Date(dump_to="startsAt")
    ends_at = fields.Date(dump_to="endsAt")
    max_executions = fields.Integer(dump_to="maxExecutions")


class SearchSubscriptionRequestSchema(BaseSchema):
    country = fields.String()
    merchant_id = fields.String(dump_to="merchantId")
    merchant_customer_id = fields.String(dump_to="merchantCustomerId")
    ssn = fields.String()
    org = fields.String()
    phone = fields.String()
    email = fields.String()
    name = fields.Nested(NameSchema)
    address = fields.Nested(AddressSchema)
    merchant_item_id = fields.String(dump_to="merchantItemId")
    subscription_state = fields.String(dump_to="subscriptionState")
    event = fields.String()
    until = fields.Date()
    from_ = fields.Date(attribute="from", dump_to="from")


class SubscriptionClient(BaseClient):
    """The client used to connect to the checkout API"""

    @property
    def stage_url(self):
        """Return the stage URL."""
        return "https://api.stage.kriita.com/subscription/" \
               "v{0}".format(self.version)

    @property
    def production_url(self):
        """Return the production URL."""
        return "https://api.kriita.com/subscription" \
               "/v{0}".format(self.version)

    def create_subscription(self, **params):
        """Create a subscription."""
        url = self.build_url("create")
        return self.make_request(url, "POST", EnvelopeSchema(), 
                                 CreateSubscriptionRequestSchema, params)

    def regret_subscription(self, subscription_id):
        """Regret a subscription."""
        url = self.build_url(str(subscription_id), "regret")
        return self.make_request(url, "GET", EnvelopeSchema())

    def update_subscription(self, subscription_id, **params):
        """Update a subscription"""
        url = self.build_url(str(subscription_id), "update")
        return self.make_request(url, "POST", EnvelopeSchema(), 
                                 UpdateSubscriptionSchema, params)

    def query_subscription(self, subscription_id):
        url = self.build_url(str(subscription_id), "query")
        return self.make_request(url, "GET", EnvelopeSchema())
    
    def search_subscription(self, **params):
        """Search for subscriptions."""
        url = self.build_url("search")
        return self.make_request(url, "POST", SearchEnvelopeSchema(), 
                                 SearchSubscriptionRequestSchema, params)


def deserialize_callback_data(data, is_json_str=False):
    """Deserialize callback data

    :param data: The actual data to deserialize.
    :param is_json_str: A boolean indicating if the the
                        passed data is a JSON or a Python object.
    """
    if is_json_str:
        data = json.loads(data)
    return CallbackEnvelope().load(data).data


class Subscription(BaseResource):
    """The subscription resource."""

    CLIENT_CLS = SubscriptionClient

    @classmethod
    def create(cls, *args, **kwargs):
        return cls.get_client().create_subscription(*args, **kwargs)

    @classmethod
    def get(cls, *args, **kwargs):
        return cls.get_client().query_subscription(*args, **kwargs)

    @classmethod
    def update(cls, *args, **kwargs):
        return cls.get_client().update(*args, **kwargs)

    @classmethod
    def search(cls, *args, **kwargs):
        return cls.get_client().search_subscription(*args, **kwargs)

    @classmethod
    def regret(cls, *args, **kwargs):
        return cls.get_client().regret_subscription(*args, **kwargs)



"""
Copyright (C) 2015 David Buresund - All Rights Reserved
Unauthorized copying of this file, via any medium is strictly prohibited
Proprietary and confidential
Written by David Buresund <david.buresund@gmail.com>, September 2015
"""
from marshmallow import fields
from .utils import BaseClient, BaseSchema, BaseResource

__all__ = ["CheckoutSession", "CheckoutClient"]


class TransactionSchema(BaseSchema):
    """The Schema representing the Transaction object"""
    currency = fields.String()
    amount = fields.Float()
    merchant_transaction_id = fields.String(dump_to="merchantTransactionId")
    merchant_item_id = fields.String(dump_to="merchantItemId")
    raw_amount = fields.Float(dump_to="rawAmount")
    number_of_items = fields.Float(dump_to="numberOfItems")
    item_unit = fields.String(dump_to="itemUnit")
    amount_per_item = fields.String(dump_to="amountPerItem")
    raw_amount_per_item = fields.String(dump_to="rawAmountPerItem")
    vat = fields.Float()
    executed_at = fields.Date(load_from="executedAt", load_only=True)
    comment = fields.String()


class SubscriptionSchema(BaseSchema):
    """The Schema representing the Subscription object"""
    currency = fields.String()
    amount = fields.Float()
    interval = fields.String()
    starts_at = fields.Date(dump_to="startsAt")
    ends_at = fields.Date(dump_to="endsAt")
    max_executions = fields.Integer(dump_to="maxExecutions")
    execution_delay = fields.Integer(dump_to="executionDelay")

    merchant_subscription_id = fields.String(dump_to="merchantSubscriptionId")
    merchant_item_id = fields.String(dump_to="merchantItemId")
    number_of_items = fields.Float(dump_to="numberOfItems")
    item_unit = fields.String(dump_to="itemUnit")
    amount_per_item = fields.String(dump_to="amountPerItem")


class NameSchema(BaseSchema):
    """The Schema representing the Name object"""
    first = fields.String()
    last = fields.String()
    org = fields.String()


class AddressSchema(BaseSchema):
    """The Schema representing the Address object"""
    street = fields.String()
    zip = fields.String()
    city = fields.String()
    region = fields.String()
    country = fields.String()
    care_of = fields.String(load_from="careOf")


class CustomerSchema(BaseSchema):
    """The Schema representing the Customer object"""
    phone = fields.String()
    ssn = fields.String()
    org = fields.String()
    email = fields.String()
    name = fields.Nested(NameSchema)
    address = fields.Nested(AddressSchema)


class PayloadSchema(BaseSchema):
    """The Schema representing the Payload object"""
    url = fields.URL()
    session_id = fields.UUID(load_from="sessionId")
    status = fields.String()


class EnvelopeSchema(BaseSchema):
    """The Schema representing the Envelope object"""
    created_at = fields.DateTime(load_from="createdAt")
    status = fields.String()
    payload = fields.Nested(PayloadSchema)
    version = fields.String()


class CallbackPayloadSchema(BaseSchema):
    """The Schema representing the CallbackPayload object"""
    session_id = fields.UUID(load_from="sessionId")
    merchant_session_id = fields.String(load_from="merchantSessionId")
    merchant_customer_id = fields.String(load_from="merchant_customer_id")
    merchant_order_id = fields.String(load_from="merchantOrderId")
    executed_at = fields.String(load_from="executedAt")
    status = fields.String()
    currency = fields.String()
    amount = fields.Float()
    reservation_id = fields.Integer(load_from="reservation_id")
    customer = fields.Nested(CustomerSchema)
    billing = fields.Nested(CustomerSchema)


class CallbackEnvelope(BaseSchema):
    """The Schema representing the CallbackEnvelope object"""
    created_at = fields.DateTime(load_from="createdAt")
    sent_at = fields.DateTime(load_from="sentAt")
    callback_id = fields.Integer(load_from="callbackId")
    version = fields.String()
    event = fields.String()
    payload = fields.Nested(CallbackPayloadSchema)


class VerifyPurchaseReplySchema(BaseSchema):
    """The Schema representing the VerifyPurchaseReply object"""
    comment = fields.String()
    status = fields.String()


class VerifyPurchaseBodySchema(BaseSchema):
    """The Schema representing the VerifyPurchaseBody object"""
    session_id = fields.UUID()
    merchant_session_id = fields.String(load_from="merchantSessionId")
    merchant_customer_id = fields.String(load_from="merchantCustomerId")
    merchant_order_id = fields.String(load_from="merchantOrderId")
    executed_at = fields.Date(load_from="executedAt")
    status = fields.String()
    currency = fields.String()
    amount = fields.Float()
    reservation_id = fields.Integer(load_from="reservationId")
    customer = fields.Nested(CustomerSchema)
    billing = fields.Nested(CustomerSchema)


# TODO: Test for all camel to snake and vice versa.
class CreateSessionRequestSchema(BaseSchema):
    """The Schema representing the CreateSessionRequest object"""
    country = fields.String()
    merchant_id = fields.String(dump_to="merchantId")
    transactions = fields.Nested(TransactionSchema, many=True)
    subscriptions = fields.Nested(SubscriptionSchema, many=True)
    redirect_target = fields.String(dump_to="redirectTarget")
    redirect_on_failure_url = fields.URL(dump_to="redirectOnFailureUrl")
    redirect_on_success_url = fields.URL(dump_to="redirectOnSuccessUrl")
    callback_on_failure_url = fields.URL(dump_to="callbackOnFailureUrl")
    callback_on_success_url = fields.URL(dump_to="callbackOnSuccessUrl")

    verify_purchase_url = fields.URL(dump_to="verifyPurchaseUrl")
    merchant_terms_url = fields.URL(dump_to="merchantTermsUrl")

    language = fields.String()

    mechant_session_id = fields.String(dump_to="merchantSessionId")
    merchant_customer_id = fields.String(dump_to="merchantCustomerId")
    phone = fields.String()
    email = fields.String()

    billing_phone = fields.String(dump_to="billingPhone")
    billing_email = fields.String(dump_to="billingEmail")
    billing_name = fields.Nested(NameSchema, dump_to="billingName")
    billing_address = fields.Nested(AddressSchema, dump_to="billingAddress")

    delivery_phone = fields.String(dump_to="deliveryPhone")
    delivery_email = fields.String(dump_to="deliveryEmail")
    delivery_name = fields.Nested(NameSchema, dump_to="deliveryName")
    delivery_address = fields.Nested(AddressSchema, dump_to="deliveryAddress")

    merchant_order_id = fields.String(dump_to="merchantOrderId")

    payment_method = fields.String(dump_to="paymentMethod")
    hold_execution = fields.Boolean(dump_to="holdExecution")


class CheckoutClient(BaseClient):
    """The client used to connect to the checkout API"""

    @property
    def url(self):
        """Return the stage or production URL, depending on the settings."""
        if self.stage:
            return self.stage_url
        else:
            return self.production_url

    @property
    def stage_url(self):
        """Return the stage URL."""
        return "https://checkout.stage.paylevo.com/v{0}".format(self.version)

    @property
    def production_url(self):
        """Return the production URL."""
        return "https://checkout.paylevo.com/v{0}".format(self.version)

    def create_session(self, **params):
        """Create a checkout session.

        :param params: The parameters to pass on to the marshmallow

                deserialization, and then pass on to the server.
        :raises: marshmallow.ValidationError if a param is invalid,
                requests.RequestException if the request fails.
        :returns: Return a dictionary with the keys "response",
                "status_code", "data" and "raw_data".
        """
        url = self.build_url("session", "create")
        return self.make_request(url, "POST", EnvelopeSchema(),
                                 CreateSessionRequestSchema, params)


class CheckoutSession(BaseResource):
    """The checkout session resource."""

    CLIENT_CLS = CheckoutClient

    @classmethod
    def create(cls, **params):
        return cls.get_client().create_session(**params)



"""
Copyright (C) 2015 David Buresund - All Rights Reserved
Unauthorized copying of this file, via any medium is strictly prohibited
Proprietary and confidential
Written by David Buresund <david.buresund@gmail.com>, September 2015
"""

import os
from setuptools import setup

__version__ = "0.1.1"


# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name="sweetpay",
    version=__version__,
    author="David Buresund",
    author_email="david.buresund@paylevo.com",
    description="A SDK to talk with the Sweetpay APIs (alpha version)",
    license="Apache 2",
    keywords="sweetpay checkout payment",
    url="https://github.com/sweetpay/sweetpay-python",
    download_url="https://github.com/sweetpay/sweetpay-"
                 "python/tarball/%s" % __version__,
    packages=["sweetpay"],
    install_requires=["marshmallow==2.9.0", "requests==2.10.0"]
)



from marshmallow import fields

from .utils import BaseClient, BaseSchema, BaseResource

__all__ = ["Creditcheck", "CreditcheckClient"]


class NameSchema(BaseSchema):
    """The v representing the Name object"""
    first = fields.String()
    last = fields.String()
    org = fields.String()


class AddressSchema(BaseSchema):
    """The Schema representing the Address object"""
    street = fields.String()
    zip = fields.String()
    city = fields.String()
    region = fields.String()
    country = fields.String()
    care_of = fields.String(load_from="careOf", dump_to="careOf")


class CustomerSchema(BaseSchema):
    """The Schema representing the Customer object"""
    customer_id = fields.Integer(load_from="customerId")
    country = fields.String()

    phone = fields.String()
    ssn = fields.String()
    org = fields.String()
    email = fields.String()
    name = fields.Nested(NameSchema)
    address = fields.Nested(AddressSchema)

    merchant_customer_id = fields.String(load_from="merchantCustomerId")


class CreditSchema(BaseSchema):
    operator_id = fields.String(load_from="operatorId")
    currency = fields.String()
    credit_limit = fields.Decimal(load_from="creditLimit")
    debt = fields.Decimal()
    statuses = fields.List(fields.String())


class LogEntrySchema(BaseSchema):
    created_at = fields.DateTime(load_from="createdAt")
    customer_id = fields.Integer(load_from="customerId")
    decision = fields.String()
    limit = fields.Decimal()
    session_id = fields.UUID(load_from="sessionId")


class CreditcheckSchema(BaseSchema):
    """The Schema representing the CreditCheck object"""
    credits = fields.List(fields.Nested(CreditSchema))
    log = fields.List(fields.Nested(LogEntrySchema))
    customer = fields.Nested(CustomerSchema)


class EnvelopeSchema(BaseSchema):
    """The Schema representing the Envelope object"""
    created_at = fields.DateTime(load_from="createdAt")
    status = fields.String()
    payload = fields.Nested(CreditcheckSchema)
    version = fields.String()


class CreditcheckRequestSchema(BaseSchema):
    """The Schema representing the CreateSessionRequest object"""
    country = fields.String()
    language = fields.String()
    merchant_id = fields.String(dump_to="merchantId")
    mechant_session_id = fields.String(dump_to="merchantSessionId")
    merchant_customer_id = fields.String(dump_to="merchantCustomerId")

    ssn = fields.String()
    org = fields.String()
    phone = fields.String()
    email = fields.String()
    name = fields.Nested(NameSchema)
    address = fields.Nested(AddressSchema)

    currency = fields.String()
    amount = fields.Decimal()

    ignore_cache = fields.Boolean(dump_to="ignoreCache")


class CreditcheckSearchRequestSchema(BaseSchema):
    country = fields.String()
    operator_id = fields.String(dump_to="operatorId")

    merchant_id = fields.String(dump_to="merchantId")
    merchant_customer_id = fields.String(dump_to="merchantCustomerId")

    email = fields.String()
    phone = fields.String()
    ssn = fields.String()
    org = fields.String()
    name = fields.String()
    address = fields.String()

    decision = fields.String()
    from_ = fields.Date(attribute="from", dump_to="from")
    until = fields.Date()


class CreditcheckClient(BaseClient):
    """The client used to connect to the creditcheck API"""

    @property
    def url(self):
        """Return the stage or production URL, depending on the settings."""
        if self.stage:
            return self.stage_url
        else:
            return self.production_url

    @property
    def stage_url(self):
        """Return the stage URL."""
        return "https://api.stage.kriita.com/creditcheck/v{0}".format(
            self.version)

    @property
    def production_url(self):
        """Return the production URL."""
        return "https://api.kriita.com/creditcheck/v{0}".format(self.version)

    def make_check(self, **params):
        """Make a credit check.

        :param params: The parameters to pass on to the marshmallow
                deserialization, and then pass on to the server.
        :raises: marshmallow.ValidationError if a param is invalid,
                requests.RequestException if the request fails.
        :returns: Return a dictionary with the keys "response",
                "status_code", "data" and "raw_data".
        """
        url = self.build_url("check")
        return self.make_request(url, "POST", EnvelopeSchema(),
                                 CreditcheckRequestSchema, params)

    def search_checks(self, **params):
        url = self.build_url("log")
        return self.make_request(url, "POST", EnvelopeSchema(),
                                 CreditcheckSearchRequestSchema, params)


class Creditcheck(BaseResource):
    """The creditcheck resource."""

    CLIENT_CLS = CreditcheckClient

    @classmethod
    def create(cls, **params):
        return cls.get_client().make_check(**params)

    @classmethod
    def search(cls, **params):
        return cls.get_client().search_checks(**params)
